package require -exact snack 2.2

console show
snack::sound s -load ex1.wav

set params {
    { no 0 -1 0 }
    { yes 0 -1 1000} 
    { no 0 5000 2000} 
    { yes 0 5000 3000} 
    { no 400 5000 4000} 
    { yes 400 5000 5000}
}


set cx 0
set cx_step 200
set cy 800
set fftlen 1024
set winlen 1024
set ::canvheight $cy
set ::canvwidth [expr $cx_step * ([llength $params]+1)]

pack [canvas .c -height $::canvheight -width $::canvwidth] -side bottom
pack [button .bClose -text Close -command exit ] -side left -expand yes -fill both
pack [button .bPS -text POSTSCRIPT -command savePostScript ] -side right -expand yes -fill both



proc makeColormapSegment { prevval prevr prevg prevb  val r g b } {
    set cmapseg {}
    set num [expr $val - $prevval]
    set kr [expr (0.0 + $r - $prevr) / (0.0 + $num)]
    set kg [expr (0.0 + $g - $prevg) / (0.0 + $num)]
    set kb [expr (0.0 + $b - $prevb) / (0.0 + $num)]
    for { set idx 1 } { $idx <= $num } { incr idx } {
	set rx [expr int($kr * $idx + $prevr) ]
	set gx [expr int($kg * $idx + $prevg) ]
	set bx [expr int($kb * $idx + $prevb) ]
	lappend cmapseg [format "\#%02x%02x%02x" $rx $gx $bx ]
    }
    return $cmapseg
}
# puts [makeColormapSegment 0 0 0 0 255 64 128 255]

# input list: { 0 { 0 0 0 } 85 { 0 0 255 } 170 { 0 255 255 } 255 { 255 255 255 } }
proc makeColormap { colortable } {
    set colormap {}
    foreach {val rgbs} $colortable {
	set r [lindex $rgbs 0]
	set g [lindex $rgbs 1]
	set b [lindex $rgbs 2]
	if { [llength $colormap] == 0 } {
	    lappend colormap [format "\#%02x%02x%02x" $r $g $b ]
	} else {
	    set colormap [concat $colormap [makeColormapSegment $prevval $prevr $prevg $prevb $val $r $g $b]]
	}
	set prevval $val
	set prevr $r
	set prevg $g
	set prevb $b
    }
    return $colormap
}

#set theColormap [makeColormap { 0 { 0 0 0 } 85 { 0 255 0 } 170 { 255 255 0 } 255 { 255 255 255 } }]
set theColormap [makeColormap {
      0 {   0   0   0 }
     53 {  55  59  62 }  
     81 {  19 122 127 }
    165 { 134 206 203 }
    200 { 195 229 231 }
    224 { 255 254 236 }
    255 { 225  40 133 }
} ]



foreach param $params {
    .c create spectrogram $cx 0 -sound s -height $cy -width $cx_step -anchor nw\
	-start [expr 2500 - [lindex $param 3] / 2] \
	-end [expr 16000-[lindex $param 3]] \
	-fftlength $fftlen -winlength $winlen \
	-logfreqplot [lindex $param 0] \
	-bottomfrequency [lindex $param 1] \
	-topfrequency [lindex $param 2] \
	-gridtspacing 0.20 \
	-gridfspacing 1000 \
	-gridcrosssize 10 \
	-brightest -20 \
        -darkest -140 \
	-colormap $theColormap \
	-debug 3
    .c create rectangle $cx 0 [expr $cx+$cx_step] $cy -fill "" -outline green
    set cx [expr $cx + $cx_step]
}


#.c create rectangle 0 0 10 10 -fill red
#.c create rectangle 0 [expr $cy-10] 10 [expr $cy] -fill red
#.c create rectangle 0 0 $::canvwidth $cy -fill "" -outline blue

proc savePostScript {} {
    update
    .c postscript -file "minspeggrid_result.ps" -width $::canvwidth -height $::canvheight -pagewidth $::canvwidth -pagex [expr $::canvwidth / 2]
}


